import typing as tp
from datetime import datetime
from decimal import Decimal as Dec


class OHLC(tp.TypedDict):
    date_time: datetime
    open: Dec
    high: Dec
    low: Dec
    close: Dec
    volume: int



import typing as tp
from decimal import Decimal as Dec


class MddLimitsResponse(tp.TypedDict):
    """Maximum Drawdown. (%)"""

    day: float | None
    week: float | None
    month: float | None
    year: float | None


CapitalLimitsResponse: tp.TypeAlias = Dec  # Capital Limits


FeeResponse: tp.TypeAlias = float  # fee (%)


TaxRateResponse: tp.TypeAlias = float  # Tax Rate (%)



import typing as tp

from stsp.protocols.limits.models import (
    CapitalLimitsResponse,
    FeeResponse,
    MddLimitsResponse,
    TaxRateResponse,
)


class LimitProtocol(tp.Protocol):
    def mdd(self) -> MddLimitsResponse:
        """Maximum Drawdown provider (%)"""

    def capital(self) -> CapitalLimitsResponse:
        """Capital limits response"""

    def fee(self) -> FeeResponse:
        """Fee provider (%)"""

    def tax_rate(self) -> TaxRateResponse:
        """Tax rate provider (%)"""



import typing as tp


class RiskProtocol(tp.Protocol):
    def is_account_risk_realized(self) -> bool:
        """Checker provide True if risk for account has been relized"""

    def is_session_risk_realized(self) -> bool:
        """Checker provide True if risk for session (traiding day) has been relized"""

    def is_securities_risk_realized(self) -> bool:
        """Checker provide True if risk for  securities has been relized"""



import typing as tp

from protocols.models.common import OHLC
from stsp.protocols.position.models import ExitPointResponse
from stsp.protocols.predict.models import PredictResponse


class ClosePositionProtocol(tp.Protocol):
    async def definition_exit_point(
        self,
        *,
        figi: str,
        currnet_ohls: OHLC,
        prediction: PredictResponse,
    ) -> ExitPointResponse:
        """Provide position exit point"""



import typing as tp
from decimal import Decimal as Dec


class EntryPointResponse(tp.TypedDict):
    price: Dec
    volume: int | None


class ExitPointResponse(tp.TypedDict):
    price: Dec
    volume: int | None



"""Protocol for predict"""

import typing as tp

from stsp.protocols.models.common import OHLC
from stsp.protocols.predict.models import PredictResponse


class PredictionProtocol(tp.Protocol):
    async def predict(
        self,
        *,
        figi: str,
        current_ohlc: OHLC,
    ) -> list[PredictResponse]:
        """Predcit next N prices"""



import typing as tp
from stsp.protocols.predict.prediction import PredictionProtocol
from stsp.protocols.position.open_position import OpenPositionProtocol
from stsp.protocols.position.close_position import ClosePositionProtocol
from stsp.protocols.risk.risk import RiskProtocol
from stsp.protocols.limits.limit import LimitProtocol


class StrategyProtocol(tp.Protocol):
    """Protocol of strategy"""

    @property
    async def name(self) -> str:
        """"""

    async def limits(self) -> LimitProtocol:
        """"""

    async def predict(self) -> PredictionProtocol:
        """"""

    async def open_position(self) -> OpenPositionProtocol:
        """"""

    async def close_position(self) -> ClosePositionProtocol:
        """"""

    async def risk(self) -> RiskProtocol:
        """"""



from stsp.protocols.limits.limit import LimitProtocol
from stsp.protocols.limits.models import (
    CapitalLimitsResponse,
    FeeResponse,
    MddLimitsResponse,
    TaxRateResponse,
)

from stsp.protocols.position.close_position import ClosePositionProtocol
from stsp.protocols.position.models import ExitPointResponse, EntryPointResponse

from stsp.protocols.position.open_position import OpenPositionProtocol

from stsp.protocols.predict.prediction import PredictionProtocol

from stsp.protocols.predict.models import PredictResponse
from stsp.protocols.risk.risk import RiskProtocol

from stsp.protocols.models.common import OHLC

__all__ = [
    LimitProtocol,
    CapitalLimitsResponse,
    FeeResponse,
    MddLimitsResponse,
    TaxRateResponse,
    ClosePositionProtocol,
    ExitPointResponse,
    OpenPositionProtocol,
    EntryPointResponse,
    PredictionProtocol,
    PredictResponse,
    RiskProtocol,
    OHLC,
]
